/*=============================================================================
libxlg
filters/base_cl.c - Basic color filters
  Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2003 - 2004
  -- License restrictions apply (Dual FAL/LGPL2)
  -- For details see doc/FAL.txt and doc/LGPL2.txt.
  $Id: base_cl.c,v 1.1 2004/04/05 19:50:14 hirogen2 Exp $
=============================================================================*/
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "xlg.h"
#include "filters/strap.h"

//-----------------------------------------------------------------------------
int xlg_flt_gray(XLG_pic *ip, XLG_pic **opp, int argc, char **argv) {
    XLG_pic *op;
    int x, y;

    if((op = *opp = xlg_pic_new(ip->sx, ip->sy)) == NULL) {
        return -errno;
    }

    for(y = 0; y < ip->sy; ++y) {
        for(x = 0; x < ip->sx; ++x) {
            xlg_pic_setpx_sw(op, x, y, xlg_pic_getpx_sw(ip, x, y));
        }
    }

    return 1;
}

int xlg_flt_gray_dumb(XLG_pic *ip, XLG_pic **opp, int argc, char **argv) {
    XLG_pic *op;
    int x, y;

    if((op = *opp = xlg_pic_new(ip->sx, ip->sy)) == NULL) {
        return -errno;
    }

    for(y = 0; y < ip->sy; ++y) {
        for(x = 0; x < ip->sx; ++x) {
            uint8_t *q = xlg_pic_getpxn(ip, x, y);
            xlg_pic_setpx_sw(op, x, y, ((q[0] + q[1] + q[2]) / 3));
        }
    }

    return 1;
}

int xlg_flt_negative(XLG_pic *ip, XLG_pic **opp, int argc, char **argv) {
    XLG_pic *op;
    int x, y;

    if((op = *opp = xlg_pic_new(ip->sx, ip->sy)) == NULL) {
        return -errno;
    }

    for(x = 0; x < ip->sx; ++x) {
        for(y = 0; y < ip->sy; ++y) {
            uint8_t *cp = xlg_pic_getpxn(ip, x, y);
            xlg_pic_setpx(op, x, y, 0xFF - cp[0], 0xFF - cp[1], 0xFF - cp[2]);
        }
    }

    return 1;
}

int xlg_flt_pinv(XLG_pic *ip, XLG_pic **opp, int argc, char **argv) {
    XLG_pic *op;
    double f;
    int x, y;

    if(argc < 1) {
        fprintf(stderr,
          "PARTIAL INVERSION\n"
          "Parameters: factor\n"
          "  factor    A value, either between -1.0 to 1.0 or\n"
          "            0 to 255. (255 == total inversion)\n"
        );
        return 0;
    }

    if(strchr(argv[0], '.')) { f = strtod(argv[0], NULL); }
    else { f = ((double)strtoul(argv[0], NULL, 0) / 127.5) - 1; }

    if((op = *opp = xlg_pic_new(ip->sx, ip->sy)) == NULL) {
        return -errno;
    }

    for(x = 0; x < ip->sx; ++x) {
        for(y = 0; y < ip->sy; ++y) {
            uint8_t *cp = xlg_pic_getpxn(ip, x, y),
             r = xlg_pv_squash(0x80 + f * (128 - cp[0])),
             g = xlg_pv_squash(0x80 + f * (128 - cp[1])),
             b = xlg_pv_squash(0x80 + f * (128 - cp[2]));
            xlg_pic_setpx(op, x, y, r, g, b);
        }
    }

    return 1;
}

int xlg_flt_solarize(XLG_pic *ip, XLG_pic **opp, int argc, char **argv) {
    XLG_pic *op;
    int t, x, y;

    if(argc < 1) {
        fprintf(stderr,
          "SOLARIZE FILTER\n"
          "Apply the Solarize command to convert all colors above a specific\n"
          "lightness value into their inverse on the 255 scale.\n\n"
          "Parameters: threshold\n"
          "  The threshold level is the lightness value above which all\n"
          "  colors are inverted.\n"
        );
        return 0;
    }

    t = strtoul(argv[0], NULL, 0);
    if((op = *opp = xlg_pic_new(ip->sx, ip->sy)) == NULL) {
        return -errno;
    }

    for(x = 0; x < ip->sx; ++x) {
        for(y = 0; y < ip->sy; ++y) {
            if(xlg_pic_getpx_sw(ip, x, y) > t) {
                uint8_t *cp = xlg_pic_getpxn(ip, x, y);
                xlg_pic_setpx(op, x, y, 0xFF - cp[0], 0xFF - cp[1],
                 0xFF - cp[2]);
            }
        }
    }

    return 1;
}

//==[ End of file ]============================================================
