/*=============================================================================
libxlg
filters/cquant.c - Color quantization
  Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2003 - 2004
  -- License restrictions apply (Dual FAL/GPL2)
  -- For details see doc/FAL.txt and doc/GPL2.txt.
=============================================================================*/
#include <errno.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include "xlg.h"
#include "filters/strap.h"

//-----------------------------------------------------------------------------
int xlg_flt_cquant(XLG_pic *ip, XLG_pic **opp, int argc, char **argv) {
    int bits, bcpy, mask = 0, x, y;
    XLG_pic *op;

    if(argc < 1) {
        fprintf(stderr,
          "COLOR QUANTIZATION FILTER\n"
          "Parameters: BITS\n"
          "  BITS    Number of significant bits to leave untouched\n"
          "          The resulting image will have at most 2^BITS*RGB colors\n"
        );
        return 0;
    }

    bits = strtoul(argv[0], NULL, 0);
    bcpy = bits;
    while(bcpy--) { mask = (mask >> 1) | 128; }

    if((op = *opp = xlg_pic_new(ip->sx, ip->sy)) == NULL) {
        return -errno;
    }

    for(y = 0; y < ip->sy; ++y) {
        for(x = 0; x < ip->sx; ++x) {
            xlg_pic_setpx_sw(op, x, y, 255.0 *
             ((xlg_pic_getpx_sw(ip, x, y) & mask) >> (8 - bits)) /
             ((1 << bits) - 1));
        }
    }

    return 1;
}

//==[ End of file ]============================================================
