/*=============================================================================
libxlg
filters/histogram.c - Histrogram filters
  Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2003 - 2004
  -- License restrictions apply (Dual FAL/LGPL2)
  -- For details see doc/FAL.txt and doc/LGPL2.txt.
  $Id: hist.c,v 1.6 2004/04/05 19:50:14 hirogen2 Exp $
=============================================================================*/
#include <errno.h>
#include <stdio.h>
#include "xlg.h"
#include "filters/strap.h"

//-----------------------------------------------------------------------------
int xlg_flt_hist_st(XLG_pic *ip, XLG_pic **opp, int argc, char **argv) {
    double mf, mf2;
    short baseline, min = -1, max = -1;
    XLG_pic *op;
    int x, y;

    if((op = *opp = xlg_pic_new(ip->sx, ip->sy)) == NULL) {
        return -errno;
    }

    // Pass I: Find lightest / darkest values
    for(x = 0; x < ip->sx; ++x) {
        for(y = 0; y < ip->sy; ++y) {
            uint8_t luma = xlg_pic_getpx_sw(ip, x, y);
            if(luma < min || min == -1) { min = luma; }
            if(luma > max || max == -1) { max = luma; }
        }
    }

    // Pass II: Calculate factors
    baseline = (min + max) / 2;
    mf = (double)(255 - baseline) / (double)(max - baseline);
    mf2 = (double)baseline / (double)(baseline - min);
    if(mf2 < mf) { mf = mf2; }

    // Pass III: Apply
    for(x = 0; x < ip->sx; ++x) {
        for(y = 0; y < ip->sy; ++y) {
            short a = xlg_pic_getpx_sw(ip, x, y);
            int r = mf * (a - baseline) + baseline;
            xlg_pic_setpx_sw(op, x, y, r);
        }
    }
    return 1;
}

//==[ End of file ]============================================================
