/*=============================================================================
libxlg
filters/matrix.c - Folding with a matrix
  Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2003 - 2004
  -- License restrictions apply (Dual FAL/LGPL2)
  -- For details see doc/FAL.txt and doc/LGPL2.txt.
  $Id: matrix.c,v 1.7 2004/04/15 12:34:55 root Exp $
=============================================================================*/
#include <errno.h>
#include <stdint.h>
#include <stdio.h>
#include "xlg.h"
#include "filters/strap.h"

static void apply_matrix(XLG_pic *, int, int, XLG_matrix *, uint8_t *);
static uint8_t apply_matrix_GS(XLG_pic *, int, int, XLG_matrix *);

//-----------------------------------------------------------------------------
int xlg_flt_matrix(XLG_pic *ip, XLG_pic **opp, int argc, char **argv) {
    XLG_matrix *mx;
    XLG_pic *op;
    int x, y;

    if(argc < 1) {
        fprintf(stderr, "Need to specify matrix name\n");
        return 0;
    }

    if((mx = xlg_mx_get(argv[0])) == NULL) {
        fprintf(stderr, "No such matrix\n");
        return 0;
    }

    if((op = *opp = xlg_pic_new(ip->sx, ip->sy)) == NULL) {
        return -errno;
    }

    for(y = 0; y < ip->sy; ++y) {
        for(x = 0; x < ip->sx; ++x) {
            uint8_t rgb[3];
            apply_matrix(ip, x, y, mx, rgb);
            xlg_pic_setpxp(op, x, y, rgb);
        }
    }
    return 1;
}

int xlg_flt_matrix_GS(XLG_pic *ip, XLG_pic **opp, int argc, char **argv) {
    XLG_matrix *mx;
    XLG_pic *op;
    int x, y;

    if(argc < 1) {
        fprintf(stderr, "Need to specify matrix name\n");
        return 0;
    }

    if((mx = xlg_mx_get(argv[0])) == NULL) {
        fprintf(stderr, "No such matrix\n");
        return 0;
    }

    if((op = *opp = xlg_pic_new(ip->sx, ip->sy)) == NULL) {
        return -errno;
    }

    for(y = 0; y < ip->sy; ++y) {
        for(x = 0; x < ip->sx; ++x) {
            xlg_pic_setpx_sw(op, x, y, apply_matrix_GS(ip, x, y, mx));
        }
    }
    return 1;
}

static void apply_matrix(XLG_pic *ip, int x, int y, XLG_matrix *mx,
 uint8_t *rgb) {
    double sr = 0, sg = 0, sb = 0;
    int r, s;

    for(r = 0; r < mx->sy; ++r) {
        for(s = 0; s < mx->sx; ++s) {
            uint8_t *cp = xlg_pic_getpxn(ip, x + s - (mx->sx / 2),
             y + r - (mx->sy / 2));
            double de = mx->table[r * mx->sy + s];
            sr += de * cp[0];
            sg += de * cp[1];
            sb += de * cp[2];
        }
    }

    sr /= mx->divisor;
    sg /= mx->divisor;
    sb /= mx->divisor;
    sr += mx->bias;
    sg += mx->bias;
    sb += mx->bias;
    rgb[0] = XLG_PV_SQUASH(sr);
    rgb[1] = XLG_PV_SQUASH(sg);
    rgb[2] = XLG_PV_SQUASH(sb);
    return;
}

static uint8_t apply_matrix_GS(XLG_pic *ip, int x, int y, XLG_matrix *mx) {
    double sr = 0;
    int r, s;

    for(r = 0; r < mx->sy; ++r) {
        for(s = 0; s < mx->sx; ++s) {
            sr += mx->table[r * mx->sy + s] *
             xlg_pic_getpx_sw(ip, x + s - (mx->sx / 2), y + r - (mx->sy / 2));
        }
    }

    sr /= mx->divisor;
    sr += mx->bias;
    return XLG_PV_SQUASH(sr);
}

//==[ End of file ]============================================================
