/*=============================================================================
libxlg
  Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2003 - 2004
  -- License restrictions apply (Dual FAL/LGPL2)
  -- For details see doc/FAL.txt and doc/LGPL2.txt.
  $Id: noise.c,v 1.6 2004/04/05 19:50:14 hirogen2 Exp $
=============================================================================*/
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include "xlg.h"
#include "filters/strap.h"

//-----------------------------------------------------------------------------
int xlg_flt_noise_add(XLG_pic *ip, XLG_pic **opp, int argc, char **argv) {
    XLG_pic *op;
    int x, y;

    if((op = *opp = xlg_pic_new(ip->sx, ip->sy)) == NULL) {
        return -errno;
    }

    for(y = 0; y < ip->sy; ++y) {
        for(x = 0; x < ip->sx; ++x) {
            int r = rand() * 50.0 / RAND_MAX;
            xlg_pic_setpx_sw(op, x, y, !r ? 0 : xlg_pic_getpx_sw(ip, x, y));
        }
    }
    return 1;
}

int xlg_flt_noise_reduc(XLG_pic *ip, XLG_pic **opp, int argc, char **argv) {
    XLG_pic *op;
    int x, y, thres;

    if(argc < 1) {
        fprintf(stderr, "noise_reduc: Specify threshold\n");
        return 0;
    }

    if((op = *opp = xlg_pic_new(ip->sx, ip->sy)) == NULL) {
        return -errno;
    }

    thres = strtoul(argv[0], NULL, 0);

    for(y = 0; y < ip->sy; ++y) {
        for(x = 0; x < ip->sx; ++x) {
            int v = 0, divisor = 0;
            uint8_t *cp;
            int p, q;
            
            for(p = -1; p <= 1; ++p) {
                for(q = -1; q <= 1; ++q) {
                    if(p == 0 && q == 0) { continue; }
                    cp = xlg_pic_getpx(ip, x + p, y + q);
                    if(cp == NULL) { continue; }
                    v += xlg_pic_getpx_sw(ip, x, y);
                    ++divisor;
                }
            }

            v /= divisor;
            if(v - xlg_pic_getpx_sw(ip, x, y) > thres) {
                xlg_pic_setpx_sw(op, x, y, v);
            }
        }
    }
    return 1;
}

//==[ End of file ]============================================================
