/*=============================================================================
libxlg
filters/resize.c - Resizing filters
  Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2004
  -- License restrictions apply (Dual FAL/LGPL2)
  -- For details see doc/FAL.txt and doc/LGPL2.txt.
  $Id: resize.c,v 1.13 2004/04/05 20:10:43 hirogen2 Exp $
=============================================================================*/
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include "xlg.h"
#include "filters/strap.h"

//-----------------------------------------------------------------------------
int xlg_flt_resize_nn(XLG_pic *ip, XLG_pic **opp, int argc, char **argv) {
    XLG_pic *op;
    int nx, ny, x, y;
    double fac;

    if(argc < 1) {
        fprintf(stderr, "Need to specify a resizing factor\n");
        return 0;
    }

    fac = strtod(argv[0], NULL);
    nx = ip->sx * fac;
    ny = ip->sy * fac;
    fac = 1 / fac;

    if((op = *opp = xlg_pic_new(nx, ny)) == NULL) {
        return -errno;
    }

    for(y = 0; y < op->sy; ++y) {
        for(x = 0; x < op->sx; ++x) {
            xlg_pic_setpxp(op, x, y, xlg_pic_getpx(ip, (double)x * fac,
             (double)y * fac));
        }
    }

    return 1;
}

int xlg_flt_resize_lx(XLG_pic *ip, XLG_pic **opp, int argc, char **argv) {
    XLG_pic *op;
    int nx, ny, dx, dy;
    double fac;

    if(argc < 1) {
        fprintf(stderr, "Need to specify a resizing factor\n");
        return 0;
    }

    fac = strtod(argv[0], NULL);
    nx = ip->sx * fac;
    ny = ip->sy * fac;
    fac = 1 / fac;

    if((op = *opp = xlg_pic_new(nx, ny)) == NULL) {
        return -errno;
    }

    for(dy = 0; dy < op->sy; ++dy) {
        for(dx = 0; dx < op->sx; ++dx) {
            double ix = (double)dx * fac;
            int iy = (double)dy * fac;
            if(ix != (int)ix) {
                int ql = ix, qr = ql + 1;
                double wl = qr - ix, wr = ix - ql;
                uint8_t *pl, *pr;
                if(qr >= ip->sx) { qr = ql; }
                pl = xlg_pic_getpx(ip, ql, iy),
                pr = xlg_pic_getpx(ip, qr, iy);
                xlg_pic_setpx(op, dx, dy,
                 XLG_PV_SQUASH(wl * pl[0] + wr * pr[0]),
                 XLG_PV_SQUASH(wl * pl[1] + wr * pr[1]),
                 XLG_PV_SQUASH(wl * pl[2] + wr * pr[2])
                );
                continue;
            }
            xlg_pic_setpxp(op, dx, dy, xlg_pic_getpx(ip, ix, iy));
        }
    }

    return 1;
}

int xlg_flt_resize_ly(XLG_pic *ip, XLG_pic **opp, int argc, char **argv) {
    XLG_pic *op;
    int nx, ny, dx, dy;
    double fac;

    if(argc < 1) {
        fprintf(stderr, "Need to specify a resizing factor\n");
        return 0;
    }

    fac = strtod(argv[0], NULL);
    nx = ip->sx * fac;
    ny = ip->sy * fac;
    fac = 1 / fac;

    if((op = *opp = xlg_pic_new(nx, ny)) == NULL) {
        return -errno;
    }

    for(dy = 0; dy < op->sy; ++dy) {
        for(dx = 0; dx < op->sx; ++dx) {
            double iy = (double)dy * fac;
            int ix = (double)dx * fac;
            if(iy != (int)iy) {
                int qu = iy, qd = qu + 1;
                double wu = qd - iy, wd = iy - qu;
                uint8_t *pu, *pd;
                if(qd >= ip->sy) { qd = qu; }
                pu = xlg_pic_getpx(ip, ix, qu),
                pd = xlg_pic_getpx(ip, ix, qd);
                xlg_pic_setpx(op, dx, dy,
                 XLG_PV_SQUASH(wu * pu[0] + wd * pd[0]),
                 XLG_PV_SQUASH(wu * pu[1] + wd * pd[1]),
                 XLG_PV_SQUASH(wu * pu[2] + wd * pd[2])
                );
                continue;
            }
            xlg_pic_setpxp(op, dx, dy, xlg_pic_getpx(ip, ix, iy));
        }
    }

    return 1;
}

int xlg_flt_resize_bi(XLG_pic *ip, XLG_pic **opp, int argc, char **argv) {
    XLG_pic *op;
    int nx, ny, dx, dy;
    double fac;

    if(argc < 1) {
        fprintf(stderr, "Need to specify a resizing factor\n");
        return 0;
    }

    fac = strtod(argv[0], NULL);
    nx = ip->sx * fac;
    ny = ip->sy * fac;
    fac = 1 / fac;

    if((op = *opp = xlg_pic_new(nx, ny)) == NULL) {
        return -errno;
    }

    for(dy = 0; dy < op->sy; ++dy) {
        for(dx = 0; dx < op->sx; ++dx) {
            double ix = (double)dx * fac, iy = (double)dy * fac;
            if(ix != (int)ix || iy != (int)iy) {
                uint8_t *s[4];
                double w[4];
                int v[4];

                v[0] = ix;
                v[1] = iy;
                v[2] = ix + 1;
                v[3] = iy + 1;

                if(v[2] >= ip->sx) { v[2] = v[0]; }
                if(v[3] >= ip->sy) { v[3] = v[1]; }

                w[0] = (ix - v[0]) * (iy - v[1]);
                w[1] = (v[2] - ix) * (iy - v[1]);
                w[2] = (ix - v[0]) * (v[3] - iy);
                w[3] = (v[2] - ix) * (v[3] - iy);

                s[0] = xlg_pic_getpx(ip, v[0], v[1]);
                s[1] = xlg_pic_getpx(ip, v[2], v[1]);
                s[2] = xlg_pic_getpx(ip, v[0], v[3]);
                s[3] = xlg_pic_getpx(ip, v[2], v[3]);

                xlg_pic_setpx(op, dx, dy,
                 XLG_PV_SQUASH(w[0] * s[0][0] + w[1] * s[1][0] + w[2] * s[2][0] + w[3] * s[3][0]),
                 XLG_PV_SQUASH(w[0] * s[0][1] + w[1] * s[1][1] + w[2] * s[2][1] + w[3] * s[3][1]),
                 XLG_PV_SQUASH(w[0] * s[0][2] + w[1] * s[1][2] + w[2] * s[2][2] + w[3] * s[3][2])
                );
                continue;
            }
            xlg_pic_setpxp(op, dx, dy, xlg_pic_getpx(ip, ix, iy));
        }
    }

    return 1;
}

//==[ End of file ]============================================================
