/*=============================================================================
libxlg
filters/robert.c - Find edges
  Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2003 - 2004
  -- License restrictions apply (Dual FAL/LGPL2)
  -- For details see doc/FAL.txt and doc/LGPL2.txt.
  $Id: robert.c,v 1.7 2004/04/05 19:50:14 hirogen2 Exp $
=============================================================================*/
#include <errno.h>
#include <math.h>
#include <stdint.h>
#include <stdio.h>
#include "xlg.h"
#include "filters/strap.h"

//-----------------------------------------------------------------------------
int xlg_flt_robert(XLG_pic *ip, XLG_pic **opp, int argc, char **argv) {
    XLG_pic *op;
    int x, y;

    if((op = *opp = xlg_pic_new(ip->sx, ip->sy)) == NULL) {
        return -errno;
    }

    for(y = 0; y < ip->sy; ++y) {
        for(x = 0; x < ip->sx; ++x) {
            uint8_t *ul = xlg_pic_getpxn(ip, x - 1, y - 1),
             *lt = xlg_pic_getpxn(ip, x - 1, y),
             *dn = xlg_pic_getpxn(ip, x, y - 1),
             *cp = xlg_pic_getpxn(ip, x, y);
            int rv = ul[0] - cp[0], rw = lt[0] - dn[0],
                gv = ul[1] - cp[1], gw = lt[1] - dn[1],
                bv = ul[2] - cp[2], bw = lt[2] - dn[2];
            xlg_pic_setpx(op, x, y, sqrt(rv * rv + rw * rw),
             sqrt(gv * gv + gw * gw), sqrt(bv * bv + bw * bw));
        }
    }

    return 1;
}

int xlg_flt_robert_GS(XLG_pic *ip, XLG_pic **opp, int argc, char **argv) {
    // Faster grayscale variant
    XLG_pic *op;
    int x, y;

    if((op = *opp = xlg_pic_new(ip->sx, ip->sy)) == NULL) {
        return -errno;
    }

    for(y = 0; y < ip->sy; ++y) {
        for(x = 0; x < ip->sx; ++x) {
            int v = xlg_pic_getpx_sw(ip, x - 1, y - 1) -
             xlg_pic_getpx_sw(ip, x, y);
            int w = xlg_pic_getpx_sw(ip, x - 1, y) -
             xlg_pic_getpx_sw(ip, x, y - 1);
            xlg_pic_setpx_sw(op, x, y, sqrt(v * v + w * w));
        }
    }

    return 1;
}

//==[ End of file ]============================================================
