/*=============================================================================
libxlg
filters/rotate.c - Free rotate
  Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2004
  -- License restrictions apply (Dual FAL/LGPL2)
  -- For details see doc/FAL.txt and doc/LGPL2.txt.
  $Id: rotate.c,v 1.8 2004/05/20 16:45:53 root Exp $
=============================================================================*/
#define _GNU_SOURCE 1 // round()
#include <errno.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "xlg.h"
#include "filters/strap.h"
#define DEG2RAD(__x) ((__x) * M_PI / 180)
#define RAD2DEG(__x) ((__x) * 180 / M_PI)

//-----------------------------------------------------------------------------
int xlg_flt_rotate(XLG_pic *ip, XLG_pic **opp, int argc, char **argv) {
    double alpha = 0, zx, zy;
    int nx, ny, x, y;
    XLG_pic *op;

    if(argc < 1) {
        fprintf(stderr, "rotate: Need to specify rotation angle\n");
        return 0;
    }

    // Get new image size
    alpha = DEG2RAD(strtod(argv[0], NULL));
    nx = sin(alpha) * ip->sy + cos(alpha) * ip->sx;
    ny = sin(alpha) * ip->sx + cos(alpha) * ip->sy;

    if((op = *opp = xlg_pic_new(nx, ny)) == NULL) {
        return -errno;
    }

    zx = nx / 2;
    zy = ny / 2;

    for(y = 0; y < op->sy; ++y) {
        for(x = 0; x < op->sx; ++x) {
            double dx = zx - x, dy = zy - y;
            double ln = sqrt(dx * dx + dy * dy);
            double ra = acos(dx / ln);
            int ox, oy;

            if(dy < 0) { ra = 2 * M_PI - ra; }
            ra -= alpha;
            ox = round(ip->sx / 2 - cos(ra) * ln);
            oy = round(ip->sy / 2 - sin(ra) * ln);
            xlg_pic_setpxp(op, x, y, xlg_pic_getpxn(ip, ox, oy));
        }
    }

    // Compensate for center hole
    xlg_pic_setpxp(op, zx, zy, xlg_pic_getpx(ip, ip->sx / 2, ip->sy / 2));
    return 1;
}

int xlg_flt_rol90(XLG_pic *ip, XLG_pic **opp, int argc, char **argv) {
    XLG_pic *op;
    int x, y;

    if((op = *opp = xlg_pic_new(ip->sy, ip->sx)) == NULL) {
        return -errno;
    }

    for(x = 0; x < ip->sx; ++x) {
        for(y = 0; y < ip->sy; ++y) {
            xlg_pic_setpxp(op, y, ip->sx - x - 1, xlg_pic_getpxn(ip, x, y));
        }
    }

    return 1;
}

int xlg_flt_ror90(XLG_pic *ip, XLG_pic **opp, int argc, char **argv) {
    XLG_pic *op;
    int x, y;

    if((op = *opp = xlg_pic_new(ip->sy, ip->sx)) == NULL) {
        return -errno;
    }

    for(x = 0; x < ip->sx; ++x) {
        for(y = 0; y < ip->sy; ++y) {
            xlg_pic_setpxp(op, ip->sy - y - 1, x, xlg_pic_getpxn(ip, x, y));
        }
    }
    return 1;
}

//==[ End of file ]============================================================
