/*=============================================================================
libxlg
matrix/mx2_doom_soft.c - Special soften matrix filter to work with image bounds
  and the Doom transparent color
  Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2004
  -- License restrictions apply (Dual FAL/LGPL2)
  -- For details see doc/FAL.txt and doc/LGPL2.txt.
  $Id: mx2_doom_soft.c,v 1.6 2004/04/15 13:54:46 root Exp $

This filter is a modification of the mx2_soft filter to recognize the color
{0, 255, 255} (= cyan) as a transparent color and thus as an image boundary.

This filter of course requires the "matrix" filter and the "soft" matrix table
to be existant.
=============================================================================*/
#include <errno.h>
#include <stdint.h>
#include <stdio.h>
#include "xlg.h"
#include "filters/strap.h"

static void apply_matrix(XLG_pic *, int, int, XLG_matrix *, uint8_t *);

//-----------------------------------------------------------------------------
int xlg_flt_matrix2_doom_soft(XLG_pic *ip, XLG_pic **opp, int argc,
 char **argv) {
    XLG_matrix *mx;
    XLG_pic *op;
    int x, y;

    if((mx = xlg_mx_get("soft")) == NULL) {
        fprintf(stderr, "mx_soft2 needs XLG_matrix:soft\n");
        return 0;
    }

    if((op = *opp = xlg_pic_new(ip->sx, ip->sy)) == NULL) {
        return -errno;
    }

    for(y = 0; y < ip->sy; ++y) {
        for(x = 0; x < ip->sx; ++x) {
            uint8_t *cp = xlg_pic_getpxn(ip, x, y), rgb[3];
            if(cp[0] == 0 && cp[1] == 255 && cp[2] == 255) { continue; }
            apply_matrix(ip, x, y, mx, rgb);
            xlg_pic_setpxp(op, x, y, rgb);
        }
    }
    return 1;
}

static void apply_matrix(XLG_pic *ip, int x, int y, XLG_matrix *mx,
 uint8_t *rgb) {
    double sr = 0, sg = 0, sb = 0;
    double dv = 0;
    int r, s;

    for(r = 0; r < mx->sy; ++r) {
        for(s = 0; s < mx->sx; ++s) {
            uint8_t *cp = xlg_pic_getpx(ip, x + s - (mx->sx / 2),
             y + r - (mx->sy / 2));
            double de;

            if(cp == NULL) { continue; }
            de = mx->table[r * mx->sy + s];

            sr += de * cp[0];
            sg += de * cp[1];
            sb += de * cp[2];
            ++dv;
        }
    }

    sr /= dv;
    sg /= dv;
    sb /= dv;
    sr += mx->bias;
    sg += mx->bias;
    sb += mx->bias;
    if(sr < 0) { sr = 0; } else if(sr > 255) { sr = 255; }
    if(sg < 0) { sg = 0; } else if(sg > 255) { sg = 255; }
    if(sb < 0) { sb = 0; } else if(sb > 255) { sb = 255; }
    rgb[0] = sr;
    rgb[1] = sg;
    rgb[2] = sb;
    return;
}

//==[ End of file ]============================================================
