
// http://www.cs.rit.edu/~ncs/color/t_convert.html
static void rgb2hsv(double r, double g, double b, double *h, double *s,
 double *v) {
    double min = MIN(r, g, b), max = MAX(r, g, b), delta = max - min;
    *v = max;

    if(max != 0) {
        *s = delta / max;
    } else {
        // r = g = b = 0 ==> s = 0, v is undefined

        //*s = 0;
        //*h = -1;

        *h = *s = *v = 0;
        return;
    }

    if(r == max) {
        *h = (g - b) / delta; // between yellow & magenta
    } else if(g == max) {
        *h = 2 + (b - r) / delta; // between cyan & yellow
    } else {
        *h = 4 + (r - g) / delta; // between magenta & cyan
    }

    *h *= 60; // degrees
    if(*h < 0) { *h += 360; }
    return;
}

static void hsv2rgb(double h, double s, double v, double *r, double *g,
 double *b) {
    int i;
    double f, p, q, t;
    if(s == 0) {
        // achromatic (grey)
        *r = *g = *b = v;
        return;
    }

    h /= 60; // sector 0 to 5
    i = h; // implicit floor(h);
    f = h - i; // factorial part of h
    p = v * (1 - s);
    q = v * (1 - s * f);
    t = v * (1 - s * (1 - f));

    switch(i) {
        case 0:
            *r = v;
            *g = t;
            *b = p;
            break;
        case 1:
            *r = q;
            *g = v;
            *b = p;
            break;
        case 2:
            *r = p;
            *g = v;
            *b = t;
            break;
        case 3:
            *r = p;
            *g = q;
            *b = v;
            break;
        case 4:
            *r = t;
            *g = p;
            *b = v;
            break;
        default: // case 5:
            *r = v;
            *g = p;
            *b = q;
            break;
    }
    return;
}

inline static double MIN(double r, double g, double b) {
    if(g < r) { r = g; }
    if(b < r) { r = b; }
    return r;
}

inline static double MAX(double r, double g, double b) {
    if(g > r) { r = g; }
    if(b > r) { r = b; }
    return r;
}
