/*=============================================================================
libxlg
xlg.h - Header file for libxlg
  Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2003 - 2004
  -- License restrictions apply (Dual FAL/LGPL2)
  -- For details see doc/FAL.txt and doc/LGPL2.txt.
  $Id: xlg.h,v 1.6 2004/05/20 16:45:51 root Exp $
=============================================================================*/
#ifndef _XLG_H
#define _XLG_H 1

#include <sys/types.h>
#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

// XLG BASE
extern int xlg_pv_squash(int);
extern double xlg_fv_squash(double);
#ifdef __GNUC__
#  define XLG_PV_SQUASH(__v) \
    ({ \
        int __tmp = (__v); \
        (__tmp < 0) ? 0 : (__tmp > 255) ? 255 : __tmp; \
    })
#  define XLG_FV_SQUASH(__v) \
    ({ \
        double __tmp = (__v); \
        (__tmp < 0.0) ? 0.0 : (__tmp > 1.0) ? 1.0 : __tmp; \
    })
#else
#    define XLG_PV_SQUASH(__v) (xlg_pv_squash(__v))
#    define XLG_FV_SQUASH(__v) (xlg_fv_squash(__v))
#endif

// XLG PICTURE
typedef struct xlg_pic {
    int sx, sy, bpp;
    long dsize;
    uint8_t *data;
} XLG_pic;

extern XLG_pic *xlg_pic_open(const char *);
extern XLG_pic *xlg_pic_new(int, int);
extern XLG_pic *xlg_pic_dup(XLG_pic *);
extern XLG_pic *xlg_pic_zero(XLG_pic *);
extern XLG_pic *xlg_pic_border(XLG_pic *);
extern uint8_t *xlg_pic_getpx(XLG_pic *, int, int);
extern uint8_t *xlg_pic_getpxn(XLG_pic *, int, int);
extern uint8_t xlg_pic_getpx_sw(XLG_pic *, int, int);
extern uint8_t *xlg_pic_setpx(XLG_pic *, int, int, uint8_t, uint8_t, uint8_t);
extern uint8_t *xlg_pic_setpxp(XLG_pic *, int, int, uint8_t *);
extern uint8_t xlg_pic_setpx_sw(XLG_pic *, int, int, uint8_t);
extern int xlg_pic_store(XLG_pic *, const char *);
extern void xlg_pic_free(XLG_pic *);

// XLG FILTER
struct xlg_filter_struct;
typedef struct xlg_filter_struct XLG_filter;

struct xlg_filter_struct {
    XLG_filter *Next;
    const char *name, *desc;
    int (*proc)(XLG_pic *, XLG_pic **, int, char **);
    int (*proc_gs)(XLG_pic *, XLG_pic **, int, char **);
    int show;
};

extern int xlg_flt_register(XLG_filter *);
extern int xlg_flt_unregister(XLG_filter *);
extern int xlg_flt_list(XLG_filter ***);
extern XLG_filter *xlg_flt_get(const char *);

// XLG MATRIX
struct xlg_matrix_struct;
typedef struct xlg_matrix_struct XLG_matrix;

struct xlg_matrix_struct {
    XLG_matrix *Next;
    const char *name, *desc;
    int sx, sy, sz;
    const double *table;
    double divisor, bias;
    int show;
};

extern int xlg_mx_register(XLG_matrix *);
extern int xlg_mx_unregister(XLG_matrix *);
extern int xlg_mx_list(XLG_matrix ***);
extern XLG_matrix *xlg_mx_get(const char *);

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _XLG_H

//==[ End of file ]============================================================
